﻿import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.InputStream;

import javax.xml.namespace.QName;
import javax.xml.xquery.XQConnection;
import javax.xml.xquery.XQConstants;
import javax.xml.xquery.XQDataSource;
import javax.xml.xquery.XQException;
import javax.xml.xquery.XQExpression;
import javax.xml.xquery.XQItemType;
import javax.xml.xquery.XQPreparedExpression;
import javax.xml.xquery.XQResultSequence;
import javax.xml.xquery.XQSequence;

import net.xqj.exist.ExistXQDataSource;

import org.w3c.dom.Element;
import org.xml.sax.ContentHandler;
import org.xml.sax.SAXException;
import org.xml.sax.helpers.DefaultHandler;
import org.xml.sax.Attributes;

public class XQJTest {
	public static XQConnection createConnection() throws XQException {
		XQDataSource xqs = new ExistXQDataSource();
		xqs.setProperty("serverName", "localhost");
		xqs.setProperty("port", "8080");
		XQConnection conn = xqs.getConnection();
		return conn;
	}

	public static XQSequence testExpressionSimple(XQConnection conn, String Q)
			throws XQException {
		XQExpression xqje = conn.createExpression();
		XQSequence xqseq = xqje.executeQuery(Q);
		return xqseq;
	}

	//Java ex. 4
	public static void testFileandContext() 
			throws FileNotFoundException, XQException {
			  InputStream query;
			  query = new FileInputStream("/home/xquery/share/xqueries/test.xq");
			  XQConnection conn = createConnection();
			  XQResultSequence xqseq = conn.createExpression().executeQuery(query);
			  while (xqseq.next()) {
			    Element secteur = (org.w3c.dom.Element) xqseq.getObject();
			    System.out.print("Secteur "+secteur.getAttribute("id")+":");
			    XQExpression xqe = conn.createExpression();
			    xqe.bindNode(XQConstants.CONTEXT_ITEM, secteur, null);
			    XQSequence xqseq2 = xqe.executeQuery("./itinéraire/nom");
			    xqseq2.writeSequence(System.out, null);
			  }
			}

	//Java ex 6
	public static void aTwoXQpipeline() throws XQException {
		  XQConnection conn = createConnection();
		  XQResultSequence xqseq1 = conn
		    .createExpression()
		    .executeQuery("fn:distinct-values(fn:doc('/db/Guide/queyras.xml')//cotation)");
		  XQExpression xqexp2 = conn.createExpression();
		  xqexp2.bindSequence(new QName("cotations"), xqseq1);
		  String e2 =
		    "declare variable $cotations as xs:untypedAtomic* external;"
		    + "<réponse> "
		    + "  {for $c in $cotations order by $c return"
		    + "     <difficulté valeur='{$c}'>"
		    + "       {for $i in"
		    + "          fn:doc('/db/Guide/queyras.xml')//itinéraire"
		    + "        where $i/*:cotation = $c"
		    + "        return <iti id='{$i/@id}'/>}"
		    + "     </difficulté>}"
		    + "</réponse>";
		  XQSequence xqseq2 = xqexp2.executeQuery(e2);
		  xqseq2.writeSequence(System.out, null);
		  xqseq2.close();
		  xqseq1.close();
		}

	
	public static void main(String[] args) throws XQException, FileNotFoundException {
		XQConnection conn = null;
		try {
			conn = createConnection();
			XQSequence xqseq = testExpressionSimple(conn,
					"fn:doc('/db/Guide/queyras.xml')//secteur/nom");
			// Java ex1
			System.out.println("-- Java 1");
			xqseq.writeSequence(System.out, null);
			System.out.println();
			
			// Java ex2
			System.out.println("-- Java 2");
			XQPreparedExpression xqpe = conn
					.prepareExpression("declare variable $u as xs:string external;"
							+ "declare variable $f as xs:decimal external;"
							+ "<altitude unité='{$u}'>{$f * 3033}</altitude>");
			xqpe.bindString(new QName("u"), "ft", null);
			xqpe.bindAtomicValue(new QName("f"), "3.2809",
					conn.createAtomicType(XQItemType.XQBASETYPE_DECIMAL));
			XQResultSequence rs = xqpe.executeQuery();
			while (rs.next())
				System.out.println(rs.getItemAsString(null));

			// Java ex3
			System.out.println("-- Java 3");
			xqseq = testExpressionSimple(conn, "(3, 4)");
			while (xqseq.next())
			  System.out.println((java.math.BigInteger) xqseq.getObject());
			xqseq = testExpressionSimple(conn, "(3.0, 4.0)");
			while (xqseq.next())
			  System.out.println((java.math.BigDecimal) xqseq.getObject());
			xqseq = testExpressionSimple(conn, "(3, 'abc', 4.0)");
			while (xqseq.next())
			  System.out.println(xqseq.getAtomicValue() + " ");

			// Java ex4
			System.out.println("-- Java 4");
			testFileandContext();
			
			// Java ex5
			System.out.println("-- Java 5");
			xqseq = testExpressionSimple(conn,
			          "fn:doc('/db/Guide/queyras.xml')//itinéraire");
			ContentHandler ch = new DefaultHandler() {
			  String id;
			  boolean inCotation = false;
			  @Override
			  public void startElement(String uri, String localName, String qName,  Attributes attributes) throws SAXException {
			    if (qName.equals("itinéraire")) id = attributes.getValue("id");
			    if (qName.equals("cotation")) inCotation = true;
			  }
			  @Override
			  public void characters(char[] ch, int start, int length)
			  throws SAXException {
			    if ("***".equals(new String(ch, start, length).trim())
			      && inCotation) System.out.println(id + " ");
			  }
			  public void endElement(String uri, String localName,
			  String qName) {
			    if (qName.equals("cotation")) inCotation = false;
			  }
			};
			while (xqseq.next()) {
			   xqseq.writeItemToSAX(ch);
			   }
			//ou plus simplement xqseq.writeSequenceToSAX(ch);

			
			// Java ex6
			System.out.println("-- Java 6");
			aTwoXQpipeline();
		} catch (XQException e) {
			e.printStackTrace();
		} finally {
			if (conn != null)
				conn.close();
		}
	}
}
